require 'ostruct'
require 'csv_export'

class CustomReportController < ApplicationController
  unloadable
  helper :compliance
  helper :group_report

  include CSVExport

  def generate
    # We should redirect to the custom report if no work remains to be done.
    # We should provide a visual indication that the work has been successfully enqueued.

    @node_group = NodeGroup.find(params[:id])
    Summarizer.compare_group_against_node(@node_group, params[:node_id])
    redirect_to :back
  end

  def _show(tab)
    @hide_change_acceptance = true
    @node_group_tab = tab
    @node_group = NodeGroup.find(params[:id])
    @custom_report = CustomReport.find_by_group_id(params[:id])
    @type_filter = params[:resource_type]
    @title_filter = params[:resource_title]

    # We make no effort to display any failure information for custom reports.
    @failure_count = 0

    if @custom_report && @custom_report.node_baselines.count == @node_group.nodes.count
      @date = @custom_report.against_baseline.date

      case tab
      when 'common' then
        params[:page] ||= 1
        params[:per_page] ||= BaselineDifference.per_page
        total_count = BaselineDifference.filter_by_resource(@type_filter, @title_filter).count(
          '1',
          :joins => [:node_baseline, :baseline_resource, :report_resource],
          :group => :resource_name,
          :conditions => {:node_baseline_id => @custom_report.node_baselines.map(&:id)},
          :having => 'count(distinct node_baselines.node_id) > 1'
        ).count
        @common_changes = WillPaginate::Collection.create(params[:page], params[:per_page], total_count) do |pager|
          pager.replace Diff.group_against_node(@node_group, @custom_report.against_baseline.node, @custom_report.against_baseline.date, pager.offset, pager.per_page, @type_filter, @title_filter)
        end
        @show_filters = true

      when 'individual' then
        @individual_changes = @node_group.nodes.sort_by(&:name).map do |node|
          OpenStruct.new(
            :node_name => node.name,
            :node_id => node.id,
            :resource_differences => BaselineDifference.scoped_by_node_baseline_id(@custom_report.node_baselines.find_by_node_id(node.id)).count
          )
        end
      end
    else
      redirect_to :controller => :group_report, :action => :common
      return
    end

    render 'node_group/show'
  end

  def common
    _show('common')
  end

  def individual
    _show('individual')
  end

  def export
    custom_report = CustomReport.find_by_group_id(params[:id], :include => :node_baselines)
    respond_to do |format|
      format.csv do
        node_ids = NodeGroup.find(params[:id]).node_ids
        date = params[:date].present? ? Date.parse(params[:date]) : Time.zone.now.to_date

        baselines = custom_report.node_baselines.all(
          :joins   => :node,
          :order   => 'name',
          :include => {
            :baseline => {
              :resources => :properties,
            },
            :differences => {
              :baseline_resource => :properties,
              :report_resource => :events,
            },
          }
        ) rescue []
        data = baselines.map do |nb|
          node = nb.node.name
          baseline = nb.reference_node.name
          Diff.collate_properties_by_resource(nb).map do |resource, properties|
            type, title = resource
            properties.map do |prop, vals|
              [
                baseline,
                node,
                type,
                title,
                prop,
                vals[:baseline] == vals[:inspected],
                vals[:baseline],
                vals[:inspected],
              ]
            end
          end
        end
        data.flatten!(2)
        send_csv :data => data, :filename => "#{date.to_s(:db)}-group-#{params[:id]}-custom.csv"
      end
    end
  end

  def show_node
    @node_group = NodeGroup.find(params[:id])
    @node = Node.find(params[:node_id])
    @custom_report = CustomReport.find_by_group_id(params[:id])
    @date = @custom_report.against_baseline.date
    @node_baseline = @custom_report.node_baselines.find_by_node_id(params[:node_id])
    @type_filter = params[:resource_type]
    @title_filter = params[:resource_title]

    params[:page] ||= 1
    params[:per_page] ||= BaselineDifference.per_page
    total_count = @node_baseline.differences.filter_by_resource(@type_filter, @title_filter).count
    @baseline_differences = WillPaginate::Collection.create(params[:page], params[:per_page], total_count) do |pager|
      pager.replace Diff.node_diff(@node_baseline, pager.offset, pager.per_page, @type_filter, @title_filter)
    end

    @node_resource_changes = []
    @hide_change_acceptance = true
    @show_filters = true
  end

  def show_conflict
    @hide_change_acceptance = true
    @node_group = NodeGroup.find(params[:id])
    @custom_report = CustomReport.find_by_group_id(params[:id])
    @date = @custom_report.against_baseline.date
    @digest     = params[:digest]
    @resource   = params[:resource]

    node_ids = @node_group.nodes.map(&:id)

    node_diffs = BaselineDifference.scoped_by_node_baseline_id(@custom_report.node_baselines.map(&:id)).all(
      :conditions => {
        :resource_status_digest => @digest,
        :resource_name => @resource
      },
      :select => '*, nodes.name AS node_name',
      :joins => { :node_baseline => :node },
      :order => 'nodes.name'
    )

    @multiple_baseline_states = node_diffs.map do |node_diff|
      baseline_resource = node_diff.baseline_resource
      report_resource   = node_diff.report_resource

      baseline_properties  = baseline_resource ? baseline_resource.properties.inject({}) { |props, prop| props.merge(prop.name => prop.value) }           : {}
      inspected_properties = report_resource   ? report_resource.events.inject({}) { |props, event| props.merge(event.property => event.previous_value) } : {}

      prop_names = baseline_properties.keys | inspected_properties.keys
      properties = {}

      prop_names.each do |prop|
        properties[prop] = {
          :baseline  => baseline_properties[prop],
          :inspected => inspected_properties[prop]
        }
      end

      {
        :name => node_diff.node_name,
        :node_id => node_diff.node_id.to_i,
        :diff_id => node_diff.id,
        :accepted => node_diff.accepted,
        :properties => properties
      }
    end

    render 'node_group/multiple_baseline_states'
  end

  def all_properties_for
    redirect_to :controller => :node_report,
                :action     => :all_properties_for,
                :diff_id    => params[:diff_id]
  end
end
